@@ -33,6 +33,7 @@ Usage:
 
 import argparse
 import asyncio
+import json
 import logging
 import logging.handlers
 import os
@@ -42,7 +43,7 @@ import time
 from pathlib import Path
 from typing import Any, Optional
 
-import pyperclip
+import pyperclip  # type: ignore[import-untyped]
 
 try:
     import readline
@@ -109,6 +110,23 @@ READLINE_HISTORY_LENGTH = 1000
 logger = logging.getLogger("basic_agent_chat_loop")
 
 
+def _serialize_for_logging(obj: Any) -> str:
+    """
+    Serialize an object to JSON string for logging, with repr() fallback.
+
+    Args:
+        obj: The object to serialize
+
+    Returns:
+        JSON string if serializable, otherwise repr() string
+    """
+    try:
+        return json.dumps(obj, indent=2, default=str)
+    except (TypeError, ValueError):
+        # Fallback to repr() for non-serializable objects
+        return repr(obj)
+
+
 def setup_logging(agent_name: str) -> bool:
     """
     Setup logging with agent-specific filename, rotation, and secure permissions.
@@ -494,6 +512,12 @@ class ChatLoop:
         # Extract agent metadata
         self.agent_metadata = extract_agent_metadata(self.agent)
 
+        # Log model detection for debugging
+        detected_model = self.agent_metadata.get("model_id", "Unknown")
+        uses_harmony = self.agent_metadata.get("uses_harmony", False)
+        logger.info(f"Model Detected: {detected_model}")
+        logger.info(f"Harmony Detected: {uses_harmony}")
+
         # Setup prompt templates directory
         self.prompts_dir = Path.home() / ".prompts"
 
@@ -605,11 +629,8 @@ class ChatLoop:
         )
 
         # Auto-detect harmony support
-        # Check both metadata (for loaded agents) and agent object itself
-        # (for dynamic agents)
-        uses_harmony_metadata = self.agent_metadata.get("uses_harmony", False)
-        uses_harmony_detected = HarmonyProcessor.detect_harmony_agent(self.agent)
-        uses_harmony = uses_harmony_metadata or uses_harmony_detected
+        # Detection already performed by extract_agent_metadata()
+        uses_harmony = self.agent_metadata.get("uses_harmony", False)
 
         # Normalize config value to handle string values from YAML
         # "auto"/None → None (auto-detect)
@@ -621,9 +642,7 @@ class ChatLoop:
 
         logger.debug(f"Harmony config value (raw): {harmony_enabled_config_raw!r}")
         logger.debug(f"Harmony config value (normalized): {harmony_enabled_config!r}")
-        logger.debug(f"Harmony from metadata: {uses_harmony_metadata}")
-        logger.debug(f"Harmony from detection: {uses_harmony_detected}")
-        logger.debug(f"Auto-detected harmony (combined): {uses_harmony}")
+        logger.debug(f"Auto-detected harmony: {uses_harmony}")
 
         # Determine if harmony should be enabled
         # None = auto-detect, True = force enable, False = force disable
@@ -661,8 +680,7 @@ class ChatLoop:
                 )
         else:
             logger.info(
-                "✗ Harmony NOT enabled "
-                "(agent not detected as harmony, config not set)"
+                "✗ Harmony NOT enabled (agent not detected as harmony, config not set)"
             )
 
     def _normalize_harmony_config(self, value: Any) -> Optional[bool]:
@@ -895,10 +913,13 @@ class ChatLoop:
                 session_id = session_info.session_id
             else:
                 # Get session metadata for display
-                session_info = self.session_manager.get_session_metadata(session_id)
-                if not session_info:
+                maybe_session_info = self.session_manager.get_session_metadata(
+                    session_id
+                )
+                if not maybe_session_info:
                     print(Colors.error(f"Session not found: {session_id}"))
                     return False
+                session_info = maybe_session_info
 
             # Load the session data
             session_data = self.session_manager.load_session(session_id)
@@ -1247,12 +1268,22 @@ class ChatLoop:
 
             first_token_received = False
 
+            # Log request payload sent to agent
+            logger.info("=" * 60)
+            logger.info("REQUEST TO AGENT:")
+            logger.info(f"Query: {_serialize_for_logging(query)}")
+            logger.info("=" * 60)
+
             # Check if agent supports streaming
             if hasattr(self.agent, "stream_async"):
                 async for event in self.agent.stream_async(query):
                     # Store last event for token extraction
                     response_obj = event
 
+                    # Log streaming event received from agent
+                    logger.info("STREAMING EVENT FROM AGENT:")
+                    logger.info(_serialize_for_logging(event))
+
                     # Stop thinking indicator on first token
                     if not first_token_received:
                         stop_thinking.set()
@@ -1309,6 +1340,11 @@ class ChatLoop:
                 )
                 response_obj = response  # Store for token extraction
 
+                # Log response received from agent
+                logger.info("RESPONSE FROM AGENT:")
+                logger.info(_serialize_for_logging(response))
+                logger.info("=" * 60)
+
                 # Stop thinking indicator
                 stop_thinking.set()
                 if thinking_task:
@@ -1330,9 +1366,15 @@ class ChatLoop:
                 else:
                     response_text.append(str(response))
 
+            # Log final response object (for streaming, this is the last event)
+            if hasattr(self.agent, "stream_async"):
+                logger.info("FINAL RESPONSE OBJECT (last streaming event):")
+                logger.info(_serialize_for_logging(response_obj))
+                logger.info("=" * 60)
+
             # Render collected response
-            full_response = "".join(response_text)
-            self.last_response = full_response
+            # Use newline separator to prevent sentences from running together
+            full_response = "\n".join(response_text)
 
             # Process through Harmony if available
             display_text = full_response
@@ -1340,10 +1382,8 @@ class ChatLoop:
                 # Debug: Log response object structure
                 logger.debug(f"Response object type: {type(response_obj)}")
                 logger.debug(f"Response object attrs: {dir(response_obj)[:20]}")
-                if hasattr(response_obj, "choices"):
-                    logger.debug(
-                        f"Response has choices: {len(response_obj.choices)}"
-                    )
+                if response_obj and hasattr(response_obj, "choices"):
+                    logger.debug(f"Response has choices: {len(response_obj.choices)}")
                     if response_obj.choices:
                         choice = response_obj.choices[0]
                         logger.debug(f"Choice type: {type(choice)}")
@@ -1364,6 +1404,9 @@ class ChatLoop:
                 if processed.get("has_tools"):
                     logger.debug("Harmony response contains tool calls")
 
+            # Store last response for copy commands (what user sees)
+            self.last_response = display_text
+
             if self.use_rich and display_text.strip() and self.console:
                 # Use rich markdown rendering
                 print()  # New line after agent name
@@ -1483,7 +1526,8 @@ class ChatLoop:
                     {
                         "timestamp": time.time(),
                         "query": query,
-                        "response": full_response,
+                        # Save what user sees (includes harmony formatting)
+                        "response": display_text,
                         "duration": duration,
                         "usage": usage_info,
                     }
